''************************
''*   MIDI expander      *
''************************
'' Author:   Andy Schenk, Insonix
'' Date:     April 2009

'' uses the PhaseMod GM Synthesizer to play the MIDI events from the RX input.


CON
  _clkmode        = xtal1 + pll16x
  _xinfreq        = 5_000_000

  SD_basepin      = 8       '<-- SD card
  Left            = 4       '<-- Audio Out pins
  Right           = 6
        

OBJ
  midi  : "FD_Serial_Conf"  'FDserial with 128 byte rx-buffer
  synth : "pm_synth_20"
  disp  : "TV_Text"         'or "VGA_Text" or "PC_Text"


PUB Main
  disp.start(12)                                        '<-- Display basepin
  midi.start(31,30,0,38400)                             'MIDI input (PropPlug)
'  midi.start(InPin,OutPin,0,31250)                      'Hardware MIDI input (set right Pins for your hardware)
  synth.start(Left,Right,2)                             'start synth with 20 voices
  disp.str(string("MIDI Expander"))
  midiplay
  repeat
    

PUB midiplay  | lg, dt, b, v, st, d1, d2, ch
  b := st := d1 := d2 := v := 0
  repeat
    b := midi.rx                                  'MIDI byte
    if b > 127 and b < $F0
      st := b                                     'new status byte
    ch := st & $0F
    d1~
    d2~
    case st & $F0                                 'decode MIDI Event
      $90: d1 := midi.rx                          'Note 
           d2 := midi.rx                          'Velocity
           if d2>0
             synth.noteOn(d1,ch,d2)               'Note On
             show(15,ch+1,string(127),-1)
           else
             synth.noteOff(d1,ch)                 'Note Off if Vel=0
             show(15,ch+1,string("."),-1)
      $80: d1 := midi.rx                          'Note Off 
           d2 := midi.rx                          'Velocity
           synth.noteOff(d1,ch)
           show(15,ch+1,string("."),-1)
           if ch==9
             show(20,ch+1,string(">"),d1)
      $A0: d1 := midi.rx                          'Poly AfterTouch 
           d2 := midi.rx                          '(not supported)
      $C0: d1 := midi.rx                          'Prg Change 
           synth.prgChange(d1,ch)
           show(0,ch+1,string("Ch:"),ch+1)
           show(6,ch+1,string("Prg:"),d1+1)
      $D0: d1 := midi.rx                          'Mono AfterTouch 
      $B0: d1 := midi.rx                          'Controller (nr) 
           d2 := midi.rx                          'value
           if d1==7
             synth.volContr(d2,ch)                '7=Vol
           if d1==10
             synth.panContr(d2,ch)                '10=Pan
           if d1==120
             synth.allOff                         'AllNotes Off
             show(15,ch+1,string(" "),-1)
      $E0: d1 := midi.rx                          'Pitch Bender 
           d2 := midi.rx                          '(not supported!)
      $F0: if st==$F0                             'SysEx
             repeat
               v := midi.rx  
             until v==$F7
             d1~
           elseif st==$F2
             d1 := midi.rx                        '3 byte
             d2 := midi.rx  

PUB show(x,y,strp,val)                                  ''visualize MIDI events
  if y<12
    disp.out(10)
    disp.out(x)
    disp.out(11)
    disp.out(y)
    disp.str(strp)
    if val=>0
      disp.dec(val)
    disp.out(" ")
     
  